package com.flyer.foster.controller.app;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.flyer.foster.dto.GardenQueryDTO;
import com.flyer.foster.dto.GardenRespDTO;
import com.flyer.foster.dto.SampleQueryDto;
import com.flyer.foster.dto.app.AppSampleAge;
import com.flyer.foster.dto.app.AppSampleSpeciesItem;
import com.flyer.foster.entity.Farm;
import com.flyer.foster.service.IGardenService;
import com.flyer.foster.service.rpc.farm.FeignFarmRegionService;
import com.flyer.foster.service.rpc.farm.FeignFarmService;
import com.flyer.foster.service.rpc.farm.model.FeignFarmDto;
import com.flyer.foster.service.rpc.farm.model.FeignRegionDto;
import com.flyer.foster.service.rpc.sample.FeignSampleService;
import com.flyer.foster.service.rpc.sample.model.FeignSampleDto;
import com.flyer.foster.service.rpc.species.FeignSpeciesItemService;
import com.flyer.foster.service.rpc.species.model.SpeciesItem;
import com.flyer.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 农场（小程序）
 *
 * @author flyer
 * @since 2024-05-09
 */
@RestController
@RequestMapping("/app/farm")
public class AppFarmController {
    @Autowired
    private IGardenService iGardenService;
    @Autowired
    private FeignFarmService feignFarmService;
    @Autowired
    private FeignFarmRegionService feignFarmRegionService;
    @Autowired
    private FeignSampleService feignSampleService;
    @Autowired
    private FeignSpeciesItemService feignSpeciesItemService;

    /**
     * 获取真实农场采样点List
     * @return
     */
    @PostMapping("/getRealFarmSampleList")
    public R getRealFarmSampleList(@RequestBody SampleQueryDto dto){
        FeignFarmDto farm = feignFarmService.get(dto.getFarmId());
        List<SpeciesItem> speciesItems = feignSpeciesItemService.listByPid(farm.getSpeciesId());
        Map<Long, SpeciesItem> map = new HashMap<>();
        for (SpeciesItem item : speciesItems) {
            map.put(item.getId(), item);
        }

        List<FeignSampleDto> list = feignSampleService.list(dto);
        Map<Long, AppSampleSpeciesItem> speciesItemMap = new HashMap<>();
        //统计品种和树龄
        for (FeignSampleDto sample : list) {
            speciesItemMap.put(sample.getSpeciesItemId(),
                    new AppSampleSpeciesItem(sample.getSpeciesItemId(),map.get(sample.getSpeciesItemId()).getName()));
        }
        //构造树龄标准
        List<AppSampleAge> ageList = new ArrayList<>(3);
        ageList.add(new AppSampleAge("1-3年", new Integer[]{0,3}));
        ageList.add(new AppSampleAge("3-5年", new Integer[]{3,5}));
        ageList.add(new AppSampleAge("超过10年", new Integer[]{10,100}));
        return R.ok().result(list).extData(new Object[]{speciesItemMap.values()
        ,ageList});
    }

    /**
     * 获取真实农场区域信息
     * @return
     */
    @PostMapping("/getRealFarmRegionList")
    public R getRealFarmRegionList(@RequestBody SampleQueryDto dto){
        List<FeignRegionDto> list = feignFarmRegionService.list(dto);
        return R.ok().result(list);
    }

    /**
     * 获取真实农场信息
     * @return
     */
    @GetMapping("/getRealFarm")
    public R getRealFarm(@RequestParam Integer farmId){
        FeignFarmDto dto = feignFarmService.get(farmId);
        return R.ok().result(dto);
    }

    /**
     * 果园信息获取
     * @return
     */
    @GetMapping("/get")
    public R get(@RequestParam Integer farmId) {
        Farm bean = iGardenService.lambdaQuery().eq(Farm::getBirdseyeFarmId, farmId).one();
        return R.ok().result(bean);
    }

    /**
     * 果园列表-分页
     *
     * @param current 当前页
     * @param size    当前页显示条数
     * @return
     */
    @GetMapping("/list/{current}/{size}")
    public R getPageList(@PathVariable Integer current, @PathVariable Integer size, GardenQueryDTO queryDTO) {
        IPage<GardenRespDTO> page = new Page<>(current, size);
        return R.ok().result(iGardenService.getPageList(page, queryDTO));
    }
}
