package com.flyer.foster.controller.app;

import com.flyer.foster.dto.TreeAddDTO;
import com.flyer.foster.dto.TreeRespDTO;
import com.flyer.foster.dto.TreeUpdateDTO;
import com.flyer.foster.entity.Sample;
import com.flyer.foster.pojo.StpAppUtil;
import com.flyer.foster.service.ITbFarmFosterCodeService;
import com.flyer.foster.service.ITreeService;
import com.flyer.foster.service.impl.TbFarmFosterCodeServiceImpl;
import com.flyer.foster.service.rpc.farm.FeignFarmWorkService;
import com.flyer.foster.service.rpc.farm.model.ZFarmWorkRecordSampleDetailVo;
import com.flyer.foster.service.rpc.sample.FeignSampleService;
import com.flyer.foster.service.rpc.sample.model.FeignSampleDto;
import com.flyer.model.PageAndListResult;
import com.flyer.util.CopyUtil;
import com.flyer.util.R;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 树（小程序）
 * @author flyer
 * @since 2024-05-09
 */
@RestController
@RequestMapping("/app/sample")
public class AppTreeController {
    @Autowired
    private ITreeService iTreeService;
    @Autowired
    private FeignFarmWorkService feignFarmWorkService;
    @Autowired
    private FeignSampleService feignSampleService;
    @Autowired
    private ITbFarmFosterCodeService tbFarmFosterCodeService;

    /**
     * 使用认养码并且保存树
     */
    @PostMapping("useFosterCodeSaveSample")
    @Transactional
    public R useFosterCodeSaveSample(TreeAddDTO dto){
        FeignSampleDto feignSampleDto = feignSampleService.getById(dto.getSampleId());
        Sample sample = new Sample();
        sample.setName(dto.getName());
        sample.setFarmId(feignSampleDto.getFarmId());
        sample.setRegionId(feignSampleDto.getRegionId());
        sample.setAppUserId(StpAppUtil.getLoginIdAsInt());
        sample.setSampleId(dto.getSampleId());
        iTreeService.save(sample);
        tbFarmFosterCodeService.useFosterCode(dto.getFosterCode());
        return R.ok();
    }


    /**
     * 修改树
     */
    @PostMapping("update")
    public R update(@RequestBody TreeUpdateDTO updateDTO) {
        Sample sample = iTreeService.getById(updateDTO.getId());
        BeanUtils.copyProperties(updateDTO, sample, CopyUtil.nullNames(updateDTO));
        iTreeService.updateById(sample);
        return R.ok().result(sample);
    }

    /**
     * 已认养的树List
     * @param farmId
     * @return
     */
    @GetMapping("/list/{farmId}")
    public R list(@PathVariable Integer farmId) {
        Integer userId = StpAppUtil.getLoginIdAsInt();
        List<Sample> list = iTreeService.lambdaQuery()
                .eq(Sample::getFarmId,farmId)
                .eq(Sample::getAppUserId ,userId).list();
        //获取成熟状态
        List<FeignSampleDto> feignSampleDtos = feignSampleService.getByIds(list.stream().map(Sample::getSampleId).collect(Collectors.toList()));
        Map<Integer, FeignSampleDto> map = feignSampleDtos.stream().collect(Collectors.toMap(FeignSampleDto::getId, item -> item));
        for(Sample sample : list){
            FeignSampleDto item =map.get(sample.getSampleId());
            if(item!=null){
                sample.setIsBeRipe(item.getIsBeRipe());
            }
        }
        return R.ok().result(list);
    }



    /**
     * 采样点农事记录
     * @param sampleId
     * @param page
     * @param limit
     * @return
     */
    @GetMapping("/listBySampleId/{page}/{limit}")
    public R listBySampleId(
            @RequestParam Integer sampleId,
            @PathVariable Integer page,
            @PathVariable Integer limit
    ){
        PageAndListResult<ZFarmWorkRecordSampleDetailVo> res = feignFarmWorkService.listBySampleId(sampleId, page, limit);
        return R.ok().result(res);
    }
}
