package com.flyer.foster.controller.app;

import com.flyer.foster.entity.TbAddress;
import com.flyer.foster.pojo.StpAppUtil;
import com.flyer.foster.service.ITbAddressService;
import com.flyer.util.CopyUtil;
import com.flyer.util.R;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;
import org.springframework.stereotype.Controller;

import java.util.List;

/**
 * 地址管理前端控制器
 * @author shuhao
 * @since 2025-03-16
 */
@RestController
@RequestMapping("/app/address")
public class TbAddressController {

    @Autowired
    private ITbAddressService tbAddressService;

    /**
     * 获取用户地址列表
     * @return
     */
    @GetMapping("/list")
    public R list(){
        List<TbAddress> res = tbAddressService.lambdaQuery().eq(TbAddress::getAppUserId, StpAppUtil.getLoginIdAsInt())
                .orderByAsc(TbAddress::getSort).list();
        return R.ok().result(res);
    }

    /**
     * 保存或更新地址信息
     * @param req
     * @return
     */
    @PostMapping("/save")
    @Transactional
    public R save(@RequestBody TbAddress req){
        TbAddress bean = null;
        Integer appUserId = StpAppUtil.getLoginIdAsInt();
        if(req.getId() == null){
            bean = new TbAddress();
        }else{
            bean = tbAddressService.getById(req.getId());
        }
        BeanUtils.copyProperties(req, bean, CopyUtil.nullNames(req));
        bean.setAppUserId(appUserId);
        tbAddressService.saveOrUpdate(bean);
        //是否设置默认地址
        if(req.getIsDefault() != null && req.getIsDefault().equals(1)){
            Integer minSort = tbAddressService.minSort(appUserId, bean.getId());
            bean.setSort(minSort);
        }
        if(bean.getSort() == null){
            bean.setSort(bean.getId());
        }
        tbAddressService.updateById(bean);
        return R.ok().result(bean);
    }
}
