package com.flyer.foster.controller.app;

import com.flyer.foster.dto.FosterDataDto;
import com.flyer.foster.entity.TbFarmFosterCode;
import com.flyer.foster.service.ITbFarmFosterCodeService;
import com.flyer.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.time.LocalDateTime;

/**
 * 认养码前端控制器
 * @author shuhao
 * @since 2025-03-14
 */
@RestController
@RequestMapping("/app/tbFarmFosterCode")
public class TbFarmFosterCodeController {

    @Autowired
    private ITbFarmFosterCodeService tbFarmFosterCodeService;

    /**
     * 创建果园寄养码
     * @param farmId
     * @param fosterType
     * @param targetId
     * @return
     */
    @GetMapping("/create")
    public R create(@RequestParam Integer farmId,
                    @RequestParam Integer fosterType,
                    @RequestParam Integer targetId){
        TbFarmFosterCode tbFarmFosterCode = new TbFarmFosterCode();
        tbFarmFosterCode.setFarmId(farmId);
        tbFarmFosterCode.setFosterCode(tbFarmFosterCodeService.generateFosterCode(farmId));
        tbFarmFosterCode.setFosterData(new FosterDataDto(targetId).toJsonString());
        tbFarmFosterCode.setFosterUse(0);
        tbFarmFosterCode.setFosterType(fosterType);
        tbFarmFosterCode.setExpTime(LocalDateTime.now().plusDays(3));
        tbFarmFosterCodeService.save(tbFarmFosterCode);
        return R.ok().result(tbFarmFosterCode);
    }

    /**
     * 返回果园寄养码信息
     * @param fosterCode
     * @return
     */
    @GetMapping("/getFosterCodeInfo")
    public R getFosterCodeInfo(
            @RequestParam String fosterCode,
            @RequestParam Integer farmId
    ){
        TbFarmFosterCode tbFarmFosterCode = tbFarmFosterCodeService.getFosterCodeByFosterCode(farmId, fosterCode);
        if(tbFarmFosterCode == null){
            throw new IllegalArgumentException("无效的认养码！");
        }
        tbFarmFosterCode.transFosterDataDto();
        return R.ok().result(tbFarmFosterCode);
    }

}
