package com.flyer.foster.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.flyer.foster.entity.TbFarmFosterCode;
import com.flyer.foster.mapper.TbFarmFosterCodeMapper;
import com.flyer.foster.service.ITbFarmFosterCodeService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Random;
import java.util.Set;
import java.util.concurrent.ThreadLocalRandom;
import java.util.stream.Collectors;

/**
 * <p>
 * 果园 服务实现类
 * </p>
 *
 * @author shuhao
 * @since 2025-03-14
 */
@Service
public class TbFarmFosterCodeServiceImpl extends ServiceImpl<TbFarmFosterCodeMapper, TbFarmFosterCode> implements ITbFarmFosterCodeService {


    @Override
    public String generateFosterCode(Integer farmId) {
        List<TbFarmFosterCode> tbFarmFosterCodes = this.lambdaQuery().select(TbFarmFosterCode::getFosterCode)
                .eq(TbFarmFosterCode::getFarmId, farmId)
                .eq(TbFarmFosterCode::getFosterUse, 0)
                .gt(TbFarmFosterCode::getExpTime, LocalDateTime.now())
                .list();

        Set<String> codes = tbFarmFosterCodes.stream().map(TbFarmFosterCode::getFosterCode).collect(Collectors.toSet());
        return generateCode(codes);
    }

    @Override
    public TbFarmFosterCode getFosterCodeByFosterCode(Integer farmId, String fosterCode) {
        return this.lambdaQuery().eq(TbFarmFosterCode::getFarmId, farmId)
                .eq(TbFarmFosterCode::getFosterCode, fosterCode)
               .eq(TbFarmFosterCode::getFosterUse, 0)
               .gt(TbFarmFosterCode::getExpTime, LocalDateTime.now())
               .last("LIMIT 1")
               .one();
    }

    @Override
    public void useFosterCode(String fosterCode) {
        this.lambdaUpdate().set(TbFarmFosterCode::getFosterUse, 1)
                .eq(TbFarmFosterCode::getFosterCode, fosterCode);
    }


    private String generateCode(Set<String> codes) {
        // 生成一个6位数的随机数字字符串
        String res = String.format("%06d", ThreadLocalRandom.current().nextInt(0, 1000000));
        // 如果生成的随机数字字符串已经存在，则重新生成
        while (codes.contains(res)) {
            res = String.format("%06d", ThreadLocalRandom.current().nextInt(0, 1000000));
        }
        return res;
    }

}


